﻿Imports System.IO
Imports System.Security

Namespace Algorithms

    Public Class ADLER_8

        Private Adler_Mod As UShort = &HD

        Public Function Compute(ByVal bytes As Byte()) As Byte
            Dim value As UShort = 1
            Dim s1 As UShort = (value And &HFF)
            Dim s2 As UShort = ((value >> 4) And &HFF)
            For Each b As Byte In bytes
                s1 = ((s1 + b) Mod Adler_Mod)
                s2 = ((s2 + s1) Mod Adler_Mod)
            Next
            value = (s2 << 4) Or s1
            Return CByte(value)
        End Function

    End Class

    Public Class ADLER_16

        Private Adler_Mod As UShort = &HFB

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim value As UShort = 1
            Dim s1 As UShort = value And &HFFFF
            Dim s2 As UShort = (value >> 8) And &HFFFF
            For Each b As Byte In bytes
                s1 = ((s1 + b) Mod Adler_Mod)
                s2 = ((s2 + s1) Mod Adler_Mod)
            Next
            value = ((s2 << 8) Or s1)
            bytes = BitConverter.GetBytes(CUShort(value))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class ADLER_32

        Private ADLER_MOD As UInteger = 65521

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim a As UInteger = 1
            Dim b As UInteger = 0
            For Each bit As Byte In Bytes
                a = (a + bit) Mod ADLER_MOD
                b = (b + a) Mod ADLER_MOD
            Next
            Dim Output As Byte() = BitConverter.GetBytes(CUInt((b << 16) Or a))
            Array.Reverse(Output)
            Return Output
        End Function

    End Class

    Public Class AP_Hash

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim hash As UInteger = &HAAAAAAAAL
            Dim i As UInteger = 0
            For Each b As Byte In bytes
                hash = hash Xor If((i And 1) = 0, ((hash << 7) Xor (b) * (hash >> 3)), (Not ((hash << 11) + (b) Xor (hash >> 5))))
                b += 1
                i += 1
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class BKDR_Hash

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim seed As UInteger = 131
            Dim hash As UInteger = 0
            For Each b As Byte In bytes
                hash = ((hash * seed) + (b))
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class BP_Hash

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim hash As UInteger = 0
            Dim i As UInteger = 0
            i = 0
            For Each b As Byte In bytes
                hash = hash << 7 Xor (b)
                b += 1
                i += 1
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CHECKSUM_8

        Public Function Compute(ByVal Bytes As Byte()) As Byte
            Dim num As Byte = 0
            For Each b As Byte In Bytes
                num = (num + b) And &HFF
            Next
            Return CByte(num)
        End Function

    End Class

    Public Class CHECKSUM_16

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim num As UShort = 0
            For Each b As Byte In Bytes
                num = (num + b) And &HFFFF
            Next
            Bytes = BitConverter.GetBytes(CUShort(num))
            Array.Reverse(Bytes)
            Return Bytes
        End Function

    End Class

    Public Class CHECKSUM_24

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim num As Integer = 0
            For Each b As Byte In Bytes
                num = (num + b) And &HFFFFFF
            Next
            Bytes = BitConverter.GetBytes(CInt(num))
            Array.Resize(Bytes, 3)
            Array.Reverse(Bytes)
            Return Bytes
        End Function

    End Class

    Public Class CHECKSUM_32

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim num As UInteger = 0
            For Each b As Byte In Bytes
                num = (num + b) And &HFFFFFFFF
            Next
            Bytes = BitConverter.GetBytes(CUInt(num))
            Array.Reverse(Bytes)
            Return Bytes
        End Function

    End Class

    Public Class CHECKSUM_40

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim num As Long = 0
            For Each b As Byte In Bytes
                num = (num + b) And &HFFFFFFFFFF
            Next
            Bytes = BitConverter.GetBytes(CLng(num))
            Array.Resize(Bytes, 5)
            Array.Reverse(Bytes)
            Return Bytes
        End Function

    End Class

    Public Class CHECKSUM_48

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim num As Long = 0
            For Each b As Byte In Bytes
                num = (num + b) And &HFFFFFFFFFFFF
            Next
            Bytes = BitConverter.GetBytes(CLng(num))
            Array.Resize(Bytes, 6)
            Array.Reverse(Bytes)
            Return Bytes
        End Function

    End Class

    Public Class CHECKSUM_56

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim num As Long = 0
            For Each b As Byte In Bytes
                num = (num + b) And &HFFFFFFFFFFFFFF
            Next
            Bytes = BitConverter.GetBytes(CLng(num))
            Array.Resize(Bytes, 7)
            Array.Reverse(Bytes)
            Return Bytes
        End Function

    End Class

    Public Class CHECKSUM_64

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim num As ULong = 0
            For Each b As Byte In Bytes
                num = (num + b) And &HFFFFFFFFFFFFFFFF
            Next
            Bytes = BitConverter.GetBytes(CULng(num))
            Array.Reverse(Bytes)
            Return Bytes
        End Function

    End Class

    Public Class CKSUM

        Private crctab() As UInteger = {&H0, &H4C11DB7, &H9823B6E, &HD4326D9, &H130476DC, &H17C56B6B, &H1A864DB2, &H1E475005, &H2608EDB8, &H22C9F00F, &H2F8AD6D6, &H2B4BCB61, &H350C9B64, &H31CD86D3, &H3C8EA00A, &H384FBDBD, &H4C11DB70, &H48D0C6C7, &H4593E01E, &H4152FDA9, &H5F15ADAC, &H5BD4B01B, &H569796C2, &H52568B75, &H6A1936C8, &H6ED82B7F, &H639B0DA6, &H675A1011, &H791D4014, &H7DDC5DA3, &H709F7B7A, &H745E66CD, &H9823B6E0L, &H9CE2AB57L, &H91A18D8EL, &H95609039L, &H8B27C03CL, &H8FE6DD8BL, &H82A5FB52L, &H8664E6E5L, &HBE2B5B58L, &HBAEA46EFL, &HB7A96036L, &HB3687D81L, &HAD2F2D84L, &HA9EE3033L, &HA4AD16EAL, &HA06C0B5DL, &HD4326D90L, &HD0F37027L, &HDDB056FEL, &HD9714B49L, &HC7361B4CL, &HC3F706FBL, &HCEB42022L, &HCA753D95L, &HF23A8028L, &HF6FB9D9FL, &HFBB8BB46L, &HFF79A6F1L, &HE13EF6F4L, &HE5FFEB43L, &HE8BCCD9AL, &HEC7DD02DL, &H34867077, &H30476DC0, &H3D044B19, &H39C556AE, &H278206AB, &H23431B1C, &H2E003DC5, &H2AC12072, &H128E9DCF, &H164F8078, &H1B0CA6A1, &H1FCDBB16, &H18AEB13, &H54BF6A4, &H808D07D, &HCC9CDCA, &H7897AB07, &H7C56B6B0, &H71159069, &H75D48DDE, &H6B93DDDB, &H6F52C06C, &H6211E6B5, &H66D0FB02, &H5E9F46BF, &H5A5E5B08, &H571D7DD1, &H53DC6066, &H4D9B3063, &H495A2DD4, &H44190B0D, &H40D816BA, &HACA5C697L, &HA864DB20L, &HA527FDF9L, &HA1E6E04EL, &HBFA1B04BL, &HBB60ADFCL, &HB6238B25L, &HB2E29692L, &H8AAD2B2FL, &H8E6C3698L, &H832F1041L, &H87EE0DF6L, &H99A95DF3L, &H9D684044L, &H902B669DL, &H94EA7B2AL, &HE0B41DE7L, &HE4750050L, &HE9362689L, &HEDF73B3EL, &HF3B06B3BL, &HF771768CL, &HFA325055L, &HFEF34DE2L, &HC6BCF05FL, &HC27DEDE8L, &HCF3ECB31L, &HCBFFD686L, &HD5B88683L, &HD1799B34L, &HDC3ABDEDL, &HD8FBA05AL, &H690CE0EE, &H6DCDFD59, &H608EDB80, &H644FC637, &H7A089632, &H7EC98B85, &H738AAD5C, &H774BB0EB, &H4F040D56, &H4BC510E1, &H46863638, &H42472B8F, &H5C007B8A, &H58C1663D, &H558240E4, &H51435D53, &H251D3B9E, &H21DC2629, &H2C9F00F0, &H285E1D47, &H36194D42, &H32D850F5, &H3F9B762C, &H3B5A6B9B, &H315D626, &H7D4CB91, &HA97ED48, &HE56F0FF, &H1011A0FA, &H14D0BD4D, &H19939B94, &H1D528623, &HF12F560EL, &HF5EE4BB9L, &HF8AD6D60L, &HFC6C70D7L, &HE22B20D2L, &HE6EA3D65L, &HEBA91BBCL, &HEF68060BL, &HD727BBB6L, &HD3E6A601L, &HDEA580D8L, &HDA649D6FL, &HC423CD6AL, &HC0E2D0DDL, &HCDA1F604L, &HC960EBB3L, &HBD3E8D7EL, &HB9FF90C9L, &HB4BCB610L, &HB07DABA7L, &HAE3AFBA2L, &HAAFBE615L, &HA7B8C0CCL, &HA379DD7BL, &H9B3660C6L, &H9FF77D71L, &H92B45BA8L, &H9675461FL, &H8832161AL, &H8CF30BADL, &H81B02D74L, &H857130C3L, &H5D8A9099, &H594B8D2E, &H5408ABF7, &H50C9B640, &H4E8EE645, &H4A4FFBF2, &H470CDD2B, &H43CDC09C, &H7B827D21, &H7F436096, &H7200464F, &H76C15BF8, &H68860BFD, &H6C47164A, &H61043093, &H65C52D24, &H119B4BE9, &H155A565E, &H18197087, &H1CD86D30, &H29F3D35, &H65E2082, &HB1D065B, &HFDC1BEC, &H3793A651, &H3352BBE6, &H3E119D3F, &H3AD08088, &H2497D08D, &H2056CD3A, &H2D15EBE3, &H29D4F654, &HC5A92679L, &HC1683BCEL, &HCC2B1D17L, &HC8EA00A0L, &HD6AD50A5L, &HD26C4D12L, &HDF2F6BCBL, &HDBEE767CL, &HE3A1CBC1L, &HE760D676L, &HEA23F0AFL, &HEEE2ED18L, &HF0A5BD1DL, &HF464A0AAL, &HF9278673L, &HFDE69BC4L, &H89B8FD09L, &H8D79E0BEL, &H803AC667L, &H84FBDBD0L, &H9ABC8BD5L, &H9E7D9662L, &H933EB0BBL, &H97FFAD0CL, &HAFB010B1L, &HAB710D06L, &HA6322BDFL, &HA2F33668L, &HBCB4666DL, &HB8757BDAL, &HB5365D03L, &HB1F740B4L}

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UInteger = 0
            Dim n As Integer = bytes.Length
            For Each b As Byte In bytes
                result = (result << 8) Xor Me.crctab((result >> 24) Xor b)
            Next
            While (n > 0)
                Dim b As Byte = n
                n >>= 8
                result = (result << 8) Xor Me.crctab((result >> 24) Xor b)
            End While
            bytes = BitConverter.GetBytes(CUInt(Not result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CKSUM_MPEG2

        Private crctab() As UInteger = {&H0, &H4C11DB7, &H9823B6E, &HD4326D9, &H130476DC, &H17C56B6B, &H1A864DB2, &H1E475005, &H2608EDB8, &H22C9F00F, &H2F8AD6D6, &H2B4BCB61, &H350C9B64, &H31CD86D3, &H3C8EA00A, &H384FBDBD, &H4C11DB70, &H48D0C6C7, &H4593E01E, &H4152FDA9, &H5F15ADAC, &H5BD4B01B, &H569796C2, &H52568B75, &H6A1936C8, &H6ED82B7F, &H639B0DA6, &H675A1011, &H791D4014, &H7DDC5DA3, &H709F7B7A, &H745E66CD, &H9823B6E0L, &H9CE2AB57L, &H91A18D8EL, &H95609039L, &H8B27C03CL, &H8FE6DD8BL, &H82A5FB52L, &H8664E6E5L, &HBE2B5B58L, &HBAEA46EFL, &HB7A96036L, &HB3687D81L, &HAD2F2D84L, &HA9EE3033L, &HA4AD16EAL, &HA06C0B5DL, &HD4326D90L, &HD0F37027L, &HDDB056FEL, &HD9714B49L, &HC7361B4CL, &HC3F706FBL, &HCEB42022L, &HCA753D95L, &HF23A8028L, &HF6FB9D9FL, &HFBB8BB46L, &HFF79A6F1L, &HE13EF6F4L, &HE5FFEB43L, &HE8BCCD9AL, &HEC7DD02DL, &H34867077, &H30476DC0, &H3D044B19, &H39C556AE, &H278206AB, &H23431B1C, &H2E003DC5, &H2AC12072, &H128E9DCF, &H164F8078, &H1B0CA6A1, &H1FCDBB16, &H18AEB13, &H54BF6A4, &H808D07D, &HCC9CDCA, &H7897AB07, &H7C56B6B0, &H71159069, &H75D48DDE, &H6B93DDDB, &H6F52C06C, &H6211E6B5, &H66D0FB02, &H5E9F46BF, &H5A5E5B08, &H571D7DD1, &H53DC6066, &H4D9B3063, &H495A2DD4, &H44190B0D, &H40D816BA, &HACA5C697L, &HA864DB20L, &HA527FDF9L, &HA1E6E04EL, &HBFA1B04BL, &HBB60ADFCL, &HB6238B25L, &HB2E29692L, &H8AAD2B2FL, &H8E6C3698L, &H832F1041L, &H87EE0DF6L, &H99A95DF3L, &H9D684044L, &H902B669DL, &H94EA7B2AL, &HE0B41DE7L, &HE4750050L, &HE9362689L, &HEDF73B3EL, &HF3B06B3BL, &HF771768CL, &HFA325055L, &HFEF34DE2L, &HC6BCF05FL, &HC27DEDE8L, &HCF3ECB31L, &HCBFFD686L, &HD5B88683L, &HD1799B34L, &HDC3ABDEDL, &HD8FBA05AL, &H690CE0EE, &H6DCDFD59, &H608EDB80, &H644FC637, &H7A089632, &H7EC98B85, &H738AAD5C, &H774BB0EB, &H4F040D56, &H4BC510E1, &H46863638, &H42472B8F, &H5C007B8A, &H58C1663D, &H558240E4, &H51435D53, &H251D3B9E, &H21DC2629, &H2C9F00F0, &H285E1D47, &H36194D42, &H32D850F5, &H3F9B762C, &H3B5A6B9B, &H315D626, &H7D4CB91, &HA97ED48, &HE56F0FF, &H1011A0FA, &H14D0BD4D, &H19939B94, &H1D528623, &HF12F560EL, &HF5EE4BB9L, &HF8AD6D60L, &HFC6C70D7L, &HE22B20D2L, &HE6EA3D65L, &HEBA91BBCL, &HEF68060BL, &HD727BBB6L, &HD3E6A601L, &HDEA580D8L, &HDA649D6FL, &HC423CD6AL, &HC0E2D0DDL, &HCDA1F604L, &HC960EBB3L, &HBD3E8D7EL, &HB9FF90C9L, &HB4BCB610L, &HB07DABA7L, &HAE3AFBA2L, &HAAFBE615L, &HA7B8C0CCL, &HA379DD7BL, &H9B3660C6L, &H9FF77D71L, &H92B45BA8L, &H9675461FL, &H8832161AL, &H8CF30BADL, &H81B02D74L, &H857130C3L, &H5D8A9099, &H594B8D2E, &H5408ABF7, &H50C9B640, &H4E8EE645, &H4A4FFBF2, &H470CDD2B, &H43CDC09C, &H7B827D21, &H7F436096, &H7200464F, &H76C15BF8, &H68860BFD, &H6C47164A, &H61043093, &H65C52D24, &H119B4BE9, &H155A565E, &H18197087, &H1CD86D30, &H29F3D35, &H65E2082, &HB1D065B, &HFDC1BEC, &H3793A651, &H3352BBE6, &H3E119D3F, &H3AD08088, &H2497D08D, &H2056CD3A, &H2D15EBE3, &H29D4F654, &HC5A92679L, &HC1683BCEL, &HCC2B1D17L, &HC8EA00A0L, &HD6AD50A5L, &HD26C4D12L, &HDF2F6BCBL, &HDBEE767CL, &HE3A1CBC1L, &HE760D676L, &HEA23F0AFL, &HEEE2ED18L, &HF0A5BD1DL, &HF464A0AAL, &HF9278673L, &HFDE69BC4L, &H89B8FD09L, &H8D79E0BEL, &H803AC667L, &H84FBDBD0L, &H9ABC8BD5L, &H9E7D9662L, &H933EB0BBL, &H97FFAD0CL, &HAFB010B1L, &HAB710D06L, &HA6322BDFL, &HA2F33668L, &HBCB4666DL, &HB8757BDAL, &HB5365D03L, &HB1F740B4L}

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim value As UInteger = &HFFFFFFFFUI
            For Each b As Byte In bytes
                value = (value << 8) Xor crctab(CUInt((value >> 24) Xor CUInt(b) And &HFF))
            Next
            bytes = BitConverter.GetBytes(CUInt(value))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_8

        Private CRC_Polynomial As Byte = &H8C
        Private CRC_InitialValue As Byte = 0

        Public Function Compute(ByVal Bytes As Byte()) As Byte
            Dim a As Byte
            Dim b As Byte
            Dim crc8 As Byte = CRC_InitialValue 'Initial
            For i As Integer = 0 To Bytes.Length - 1
                b = Bytes(i)
                For j As Integer = 0 To 7
                    a = 1 And (b Xor crc8)
                    crc8 = Fix(crc8 / 2) And &HFF
                    b = Fix(b / 2) And &HFF
                    If a <> 0 Then
                        crc8 = crc8 Xor CRC_Polynomial 'Poly
                    End If
                Next j
            Next
            Return CByte(crc8)
        End Function

    End Class

    Public Class CRC_16

        Private CRC_Table(255) As UShort
        Private CRC_Polynomial As UShort = &HA001
        Private CRC_InitialValue As UShort = 0

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc As UShort
            Dim c As UShort
            For i = 0 To 255
                crc = 0
                c = CUShort(i)
                For j = 0 To 7
                    If ((crc Xor c) And &H1) Then
                        crc = (crc >> 1) Xor Me.CRC_Polynomial
                    Else
                        crc = (crc >> 1)
                    End If
                    c = (c >> 1)
                Next j
                Me.CRC_Table(i) = crc
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UShort = Me.CRC_InitialValue
            Dim sum1 As UShort
            Dim sum2 As UShort
            For Each b As Byte In bytes
                sum2 = (&HFF And CUShort(b))
                sum1 = (result Xor sum2)
                result = (result >> 8) Xor Me.CRC_Table(sum1 And &HFF)
            Next
            bytes = BitConverter.GetBytes(CUShort(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_CCITT

        Private CRC_Table(255) As UShort
        Private CRC_Polynomial As UShort = &H1021
        Private CRC_InitialValue As UShort = 0

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc As UShort
            Dim c As UShort
            For i = 0 To 255
                crc = 0
                c = (CUShort(i) << 8)
                For j = 0 To 7
                    If (crc Xor c) And &H8000 Then
                        crc = (crc << 1) Xor Me.CRC_Polynomial
                    Else
                        crc = (crc << 1)
                    End If
                    c = (c << 1)
                Next j
                CRC_Table(i) = crc
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UShort = Me.CRC_InitialValue
            For Each b As Byte In bytes
                Dim sum1 As UShort
                Dim sum2 As UShort
                sum2 = (&HFF And CUShort(b))
                sum1 = (result >> 8) Xor sum2
                result = (result << 8) Xor Me.CRC_Table(sum1)
            Next
            bytes = BitConverter.GetBytes(CUShort(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_CCITT_1D0F

        Private CRC_Table(255) As UShort
        Private CRC_Polynomial As UShort = &H1021
        Private CRC_InitialValue As UShort = &H1D0F

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc As UShort
            Dim c As UShort
            For i = 0 To 255
                crc = 0
                c = (CUShort(i) << 8)
                For j = 0 To 7
                    If (crc Xor c) And &H8000 Then
                        crc = (crc << 1) Xor Me.CRC_Polynomial
                    Else
                        crc = (crc << 1)
                    End If
                    c = (c << 1)
                Next j
                CRC_Table(i) = crc
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UShort = Me.CRC_InitialValue
            For Each b As Byte In bytes
                Dim sum1 As UShort
                Dim sum2 As UShort
                sum2 = (&HFF And CUShort(b))
                sum1 = (result >> 8) Xor sum2
                result = (result << 8) Xor Me.CRC_Table(sum1)
            Next
            bytes = BitConverter.GetBytes(CUShort(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_CCITT_FFFF

        Private CRC_Table(255) As UShort
        Private CRC_Polynomial As UShort = &H1021
        Private CRC_InitialValue As UShort = &HFFFF

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc As UShort
            Dim c As UShort
            For i = 0 To 255
                crc = 0
                c = (CUShort(i) << 8)
                For j = 0 To 7
                    If (crc Xor c) And &H8000 Then
                        crc = (crc << 1) Xor Me.CRC_Polynomial
                    Else
                        crc = (crc << 1)
                    End If
                    c = (c << 1)
                Next j
                CRC_Table(i) = crc
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UShort = Me.CRC_InitialValue
            For Each b As Byte In bytes
                Dim sum1 As UShort
                Dim sum2 As UShort
                sum2 = (&HFF And CUShort(b))
                sum1 = (result >> 8) Xor sum2
                result = (result << 8) Xor Me.CRC_Table(sum1)
            Next
            bytes = BitConverter.GetBytes(CUShort(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_CCITT_XMODEM

        Private crctab() As UShort = {&H0, &H17CE, &HFDF, &H1811, &H1FBE, &H870, &H1061, &H7AF, &H1F3F, &H8F1, &H10E0, &H72E, &H81, &H174F, &HF5E, &H1890, &H1E3D, &H9F3, &H11E2, &H62C, &H183, &H164D, &HE5C, &H1992, &H102, &H16CC, &HEDD, &H1913, &H1EBC, &H972, &H1163, &H6AD, &H1C39, &HBF7, &H13E6, &H428, &H387, &H1449, &HC58, &H1B96, &H306, &H14C8, &HCD9, &H1B17, &H1CB8, &HB76, &H1367, &H4A9, &H204, &H15CA, &HDDB, &H1A15, &H1DBA, &HA74, &H1265, &H5AB, &H1D3B, &HAF5, &H12E4, &H52A, &H285, &H154B, &HD5A, &H1A94, &H1831, &HFFF, &H17EE, &H20, &H78F, &H1041, &H850, &H1F9E, &H70E, &H10C0, &H8D1, &H1F1F, &H18B0, &HF7E, &H176F, &HA1, &H60C, &H11C2, &H9D3, &H1E1D, &H19B2, &HE7C, &H166D, &H1A3, &H1933, &HEFD, &H16EC, &H122, &H68D, &H1143, &H952, &H1E9C, &H408, &H13C6, &HBD7, &H1C19, &H1BB6, &HC78, &H1469, &H3A7, &H1B37, &HCF9, &H14E8, &H326, &H489, &H1347, &HB56, &H1C98, &H1A35, &HDFB, &H15EA, &H224, &H58B, &H1245, &HA54, &H1D9A, &H50A, &H12C4, &HAD5, &H1D1B, &H1AB4, &HD7A, &H156B, &H2A5, &H1021, &H7EF, &H1FFE, &H830, &HF9F, &H1851, &H40, &H178E, &HF1E, &H18D0, &HC1, &H170F, &H10A0, &H76E, &H1F7F, &H8B1, &HE1C, &H19D2, &H1C3, &H160D, &H11A2, &H66C, &H1E7D, &H9B3, &H1123, &H6ED, &H1EFC, &H932, &HE9D, &H1953, &H142, &H168C, &HC18, &H1BD6, &H3C7, &H1409, &H13A6, &H468, &H1C79, &HBB7, &H1327, &H4E9, &H1CF8, &HB36, &HC99, &H1B57, &H346, &H1488, &H1225, &H5EB, &H1DFA, &HA34, &HD9B, &H1A55, &H244, &H158A, &HD1A, &H1AD4, &H2C5, &H150B, &H12A4, &H56A, &H1D7B, &HAB5, &H810, &H1FDE, &H7CF, &H1001, &H17AE, &H60, &H1871, &HFBF, &H172F, &HE1, &H18F0, &HF3E, &H891, &H1F5F, &H74E, &H1080, &H162D, &H1E3, &H19F2, &HE3C, &H993, &H1E5D, &H64C, &H1182, &H912, &H1EDC, &H6CD, &H1103, &H16AC, &H162, &H1973, &HEBD, &H1429, &H3E7, &H1BF6, &HC38, &HB97, &H1C59, &H448, &H1386, &HB16, &H1CD8, &H4C9, &H1307, &H14A8, &H366, &H1B77, &HCB9, &HA14, &H1DDA, &H5CB, &H1205, &H15AA, &H264, &H1A75, &HDBB, &H152B, &H2E5, &H1AF4, &HD3A, &HA95, &H1D5B, &H54A, &H1284}

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim value As UShort = 0
            For Each b As Byte In bytes
                value = (value >> 8) Xor crctab((value Xor b) And &HFF)
            Next
            bytes = BitConverter.GetBytes(CUShort(value))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_CCITT_ZMODEM

        Private crctab() As UShort = {&H0, &H1021, &H2042, &H3063, &H4084, &H50A5, &H60C6, &H70E7, &H8108, &H9129, &HA14A, &HB16B, &HC18C, &HD1AD, &HE1CE, &HF1EF, &H1231, &H210, &H3273, &H2252, &H52B5, &H4294, &H72F7, &H62D6, &H9339, &H8318, &HB37B, &HA35A, &HD3BD, &HC39C, &HF3FF, &HE3DE, &H2462, &H3443, &H420, &H1401, &H64E6, &H74C7, &H44A4, &H5485, &HA56A, &HB54B, &H8528, &H9509, &HE5EE, &HF5CF, &HC5AC, &HD58D, &H3653, &H2672, &H1611, &H630, &H76D7, &H66F6, &H5695, &H46B4, &HB75B, &HA77A, &H9719, &H8738, &HF7DF, &HE7FE, &HD79D, &HC7BC, &H48C4, &H58E5, &H6886, &H78A7, &H840, &H1861, &H2802, &H3823, &HC9CC, &HD9ED, &HE98E, &HF9AF, &H8948, &H9969, &HA90A, &HB92B, &H5AF5, &H4AD4, &H7AB7, &H6A96, &H1A71, &HA50, &H3A33, &H2A12, &HDBFD, &HCBDC, &HFBBF, &HEB9E, &H9B79, &H8B58, &HBB3B, &HAB1A, &H6CA6, &H7C87, &H4CE4, &H5CC5, &H2C22, &H3C03, &HC60, &H1C41, &HEDAE, &HFD8F, &HCDEC, &HDDCD, &HAD2A, &HBD0B, &H8D68, &H9D49, &H7E97, &H6EB6, &H5ED5, &H4EF4, &H3E13, &H2E32, &H1E51, &HE70, &HFF9F, &HEFBE, &HDFDD, &HCFFC, &HBF1B, &HAF3A, &H9F59, &H8F78, &H9188, &H81A9, &HB1CA, &HA1EB, &HD10C, &HC12D, &HF14E, &HE16F, &H1080, &HA1, &H30C2, &H20E3, &H5004, &H4025, &H7046, &H6067, &H83B9, &H9398, &HA3FB, &HB3DA, &HC33D, &HD31C, &HE37F, &HF35E, &H2B1, &H1290, &H22F3, &H32D2, &H4235, &H5214, &H6277, &H7256, &HB5EA, &HA5CB, &H95A8, &H8589, &HF56E, &HE54F, &HD52C, &HC50D, &H34E2, &H24C3, &H14A0, &H481, &H7466, &H6447, &H5424, &H4405, &HA7DB, &HB7FA, &H8799, &H97B8, &HE75F, &HF77E, &HC71D, &HD73C, &H26D3, &H36F2, &H691, &H16B0, &H6657, &H7676, &H4615, &H5634, &HD94C, &HC96D, &HF90E, &HE92F, &H99C8, &H89E9, &HB98A, &HA9AB, &H5844, &H4865, &H7806, &H6827, &H18C0, &H8E1, &H3882, &H28A3, &HCB7D, &HDB5C, &HEB3F, &HFB1E, &H8BF9, &H9BD8, &HABBB, &HBB9A, &H4A75, &H5A54, &H6A37, &H7A16, &HAF1, &H1AD0, &H2AB3, &H3A92, &HFD2E, &HED0F, &HDD6C, &HCD4D, &HBDAA, &HAD8B, &H9DE8, &H8DC9, &H7C26, &H6C07, &H5C64, &H4C45, &H3CA2, &H2C83, &H1CE0, &HCC1, &HEF1F, &HFF3E, &HCF5D, &HDF7C, &HAF9B, &HBFBA, &H8FD9, &H9FF8, &H6E17, &H7E36, &H4E55, &H5E74, &H2E93, &H3EB2, &HED1, &H1EF0}

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim value As UShort = 0
            For Each b As Byte In bytes
                value = crctab(CByte((value >> 8) Xor b)) Xor (value << 8)
            Next
            bytes = BitConverter.GetBytes(CUShort(value))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_DNP

        Private CRC_Table(255) As UShort
        Private CRC_Polynomial As UShort = &HA6BC
        Private CRC_InitialValue As UShort = 0

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc As UShort
            Dim c As UShort
            For i = 0 To 255
                crc = 0
                c = CUShort(i)
                For j = 0 To 7
                    If ((crc Xor c) And &H1) Then
                        crc = (crc >> 1) Xor Me.CRC_Polynomial
                    Else
                        crc = (crc >> 1)
                    End If
                    c = (c >> 1)
                Next j
                Me.CRC_Table(i) = crc
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UShort = Me.CRC_InitialValue
            For Each b As Byte In bytes
                Dim sum1 As UShort
                Dim sum2 As UShort
                sum2 = (&HFF And CUShort(b))
                sum1 = (result Xor sum2)
                result = (result >> 8) Xor Me.CRC_Table(sum1 And &HFF)
            Next

            result = Not result
            Dim low_byte As UShort = (result And &HFF00) >> 8
            Dim high_byte As UShort = (result And &HFF) << 8
            result = low_byte Or high_byte
            bytes = BitConverter.GetBytes(CUShort(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_IBM

        Private crctab() As UShort = {&H0, &H8005, &H800F, &HA, &H801B, &H1E, &H14, &H8011, &H8033, &H36, &H3C, &H8039, &H28, &H802D, &H8027, &H22, &H8063, &H66, &H6C, &H8069, &H78, &H807D, &H8077, &H72, &H50, &H8055, &H805F, &H5A, &H804B, &H4E, &H44, &H8041, &H80C3, &HC6, &HCC, &H80C9, &HD8, &H80DD, &H80D7, &HD2, &HF0, &H80F5, &H80FF, &HFA, &H80EB, &HEE, &HE4, &H80E1, &HA0, &H80A5, &H80AF, &HAA, &H80BB, &HBE, &HB4, &H80B1, &H8093, &H96, &H9C, &H8099, &H88, &H808D, &H8087, &H82, &H8183, &H186, &H18C, &H8189, &H198, &H819D, &H8197, &H192, &H1B0, &H81B5, &H81BF, &H1BA, &H81AB, &H1AE, &H1A4, &H81A1, &H1E0, &H81E5, &H81EF, &H1EA, &H81FB, &H1FE, &H1F4, &H81F1, &H81D3, &H1D6, &H1DC, &H81D9, &H1C8, &H81CD, &H81C7, &H1C2, &H140, &H8145, &H814F, &H14A, &H815B, &H15E, &H154, &H8151, &H8173, &H176, &H17C, &H8179, &H168, &H816D, &H8167, &H162, &H8123, &H126, &H12C, &H8129, &H138, &H813D, &H8137, &H132, &H110, &H8115, &H811F, &H11A, &H810B, &H10E, &H104, &H8101, &H8303, &H306, &H30C, &H8309, &H318, &H831D, &H8317, &H312, &H330, &H8335, &H833F, &H33A, &H832B, &H32E, &H324, &H8321, &H360, &H8365, &H836F, &H36A, &H837B, &H37E, &H374, &H8371, &H8353, &H356, &H35C, &H8359, &H348, &H834D, &H8347, &H342, &H3C0, &H83C5, &H83CF, &H3CA, &H83DB, &H3DE, &H3D4, &H83D1, &H83F3, &H3F6, &H3FC, &H83F9, &H3E8, &H83ED, &H83E7, &H3E2, &H83A3, &H3A6, &H3AC, &H83A9, &H3B8, &H83BD, &H83B7, &H3B2, &H390, &H8395, &H839F, &H39A, &H838B, &H38E, &H384, &H8381, &H280, &H8285, &H828F, &H28A, &H829B, &H29E, &H294, &H8291, &H82B3, &H2B6, &H2BC, &H82B9, &H2A8, &H82AD, &H82A7, &H2A2, &H82E3, &H2E6, &H2EC, &H82E9, &H2F8, &H82FD, &H82F7, &H2F2, &H2D0, &H82D5, &H82DF, &H2DA, &H82CB, &H2CE, &H2C4, &H82C1, &H8243, &H246, &H24C, &H8249, &H258, &H825D, &H8257, &H252, &H270, &H8275, &H827F, &H27A, &H826B, &H26E, &H264, &H8261, &H220, &H8225, &H822F, &H22A, &H823B, &H23E, &H234, &H8231, &H8213, &H216, &H21C, &H8219, &H208, &H820D, &H8207, &H202}

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim value As UShort = 0
            For Each b As Byte In bytes
                value = crctab(CByte((value >> 8) Xor b)) Xor (value << 8)
            Next
            bytes = BitConverter.GetBytes(CUShort(value))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_KERMIT

        Private CRC_Table(255) As UShort
        Private CRC_Polynomial As UShort = &H8408
        Private CRC_InitialValue As UShort = 0

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc As UShort
            Dim c As UShort
            For i = 0 To 255
                crc = 0
                c = CUShort(i)
                For j = 0 To 7
                    If ((crc Xor c) And &H1) Then
                        crc = (crc >> 1) Xor Me.CRC_Polynomial
                    Else
                        crc = (crc >> 1)
                    End If
                    c = (c >> 1)
                Next j
                Me.CRC_Table(i) = crc
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UShort = Me.CRC_InitialValue
            For Each b As Byte In bytes
                Dim sum1 As UShort
                Dim sum2 As UShort
                sum2 = (&HFF And CUShort(b))
                sum1 = (result Xor sum2)
                result = (result >> 8) Xor Me.CRC_Table(sum1 And &HFF)
            Next
            Dim low_byte As UShort = (result And &HFF00) >> 8
            Dim high_byte As UShort = (result And &HFF) << 8
            result = low_byte Or high_byte
            bytes = BitConverter.GetBytes(CUShort(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_MODBUS

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim J As Long, Odd As Boolean
            Dim crc As Integer = -1
            For Each b As Byte In bytes
                crc = (crc And &HFF00) Or ((crc And 255) Xor b)
                For J = 1 To 8
                    Odd = crc And 1
                    crc = ((crc And &HFFFE) \ 2) And &H7FFF 'shift right
                    If Odd Then crc = &HA001 Xor crc
                Next
            Next
            bytes = BitConverter.GetBytes(CUShort(crc))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_SICK

        Private CRC_Polynomial As UShort = &H8005
        Private CRC_InitialValue As UShort = 0

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UShort = Me.CRC_InitialValue
            Dim prev_byte As Byte = 0
            For Each b As Byte In bytes
                Dim short_c As UShort
                Dim short_p As UShort
                short_c = &HFF And CUShort(b)
                short_p = (&HFF And CUShort(prev_byte)) << 8
                If (result And &H8000) <> 0 Then
                    result = (result << 1) Xor Me.CRC_Polynomial
                Else
                    result = (result << 1)
                End If
                result = (result And &HFFFF)
                result = (result Xor (short_c Or short_p))

                prev_byte = b
            Next

            Dim low_byte As UShort = (result And &HFF00) >> 8
            Dim high_byte As UShort = (result And &HFF) << 8
            result = low_byte Or high_byte
            bytes = BitConverter.GetBytes(CUShort(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_16_X25

        Private crctab() As UShort = {&H0, &H1189, &H2312, &H329B, &H4624, &H57AD, &H6536, &H74BF, &H8C48, &H9DC1, &HAF5A, &HBED3, &HCA6C, &HDBE5, &HE97E, &HF8F7, &H1081, &H108, &H3393, &H221A, &H56A5, &H472C, &H75B7, &H643E, &H9CC9, &H8D40, &HBFDB, &HAE52, &HDAED, &HCB64, &HF9FF, &HE876, &H2102, &H308B, &H210, &H1399, &H6726, &H76AF, &H4434, &H55BD, &HAD4A, &HBCC3, &H8E58, &H9FD1, &HEB6E, &HFAE7, &HC87C, &HD9F5, &H3183, &H200A, &H1291, &H318, &H77A7, &H662E, &H54B5, &H453C, &HBDCB, &HAC42, &H9ED9, &H8F50, &HFBEF, &HEA66, &HD8FD, &HC974, &H4204, &H538D, &H6116, &H709F, &H420, &H15A9, &H2732, &H36BB, &HCE4C, &HDFC5, &HED5E, &HFCD7, &H8868, &H99E1, &HAB7A, &HBAF3, &H5285, &H430C, &H7197, &H601E, &H14A1, &H528, &H37B3, &H263A, &HDECD, &HCF44, &HFDDF, &HEC56, &H98E9, &H8960, &HBBFB, &HAA72, &H6306, &H728F, &H4014, &H519D, &H2522, &H34AB, &H630, &H17B9, &HEF4E, &HFEC7, &HCC5C, &HDDD5, &HA96A, &HB8E3, &H8A78, &H9BF1, &H7387, &H620E, &H5095, &H411C, &H35A3, &H242A, &H16B1, &H738, &HFFCF, &HEE46, &HDCDD, &HCD54, &HB9EB, &HA862, &H9AF9, &H8B70, &H8408, &H9581, &HA71A, &HB693, &HC22C, &HD3A5, &HE13E, &HF0B7, &H840, &H19C9, &H2B52, &H3ADB, &H4E64, &H5FED, &H6D76, &H7CFF, &H9489, &H8500, &HB79B, &HA612, &HD2AD, &HC324, &HF1BF, &HE036, &H18C1, &H948, &H3BD3, &H2A5A, &H5EE5, &H4F6C, &H7DF7, &H6C7E, &HA50A, &HB483, &H8618, &H9791, &HE32E, &HF2A7, &HC03C, &HD1B5, &H2942, &H38CB, &HA50, &H1BD9, &H6F66, &H7EEF, &H4C74, &H5DFD, &HB58B, &HA402, &H9699, &H8710, &HF3AF, &HE226, &HD0BD, &HC134, &H39C3, &H284A, &H1AD1, &HB58, &H7FE7, &H6E6E, &H5CF5, &H4D7C, &HC60C, &HD785, &HE51E, &HF497, &H8028, &H91A1, &HA33A, &HB2B3, &H4A44, &H5BCD, &H6956, &H78DF, &HC60, &H1DE9, &H2F72, &H3EFB, &HD68D, &HC704, &HF59F, &HE416, &H90A9, &H8120, &HB3BB, &HA232, &H5AC5, &H4B4C, &H79D7, &H685E, &H1CE1, &HD68, &H3FF3, &H2E7A, &HE70E, &HF687, &HC41C, &HD595, &HA12A, &HB0A3, &H8238, &H93B1, &H6B46, &H7ACF, &H4854, &H59DD, &H2D62, &H3CEB, &HE70, &H1FF9, &HF78F, &HE606, &HD49D, &HC514, &HB1AB, &HA022, &H92B9, &H8330, &H7BC7, &H6A4E, &H58D5, &H495C, &H3DE3, &H2C6A, &H1EF1, &HF78}

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim value As UShort = &HFFFFUS
            For Each b As Byte In bytes
                value = (value >> 8) Xor crctab((value Xor b) And &HFF)
            Next
            bytes = BitConverter.GetBytes(CUShort(Not value))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_24

        Private CRC24_INIT As UInteger = &HB704CE
        Private CRC24_POLY As UInteger = &H1864CFB

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim ulCRC As UInteger = CRC24_INIT
            Dim stream As New MemoryStream(Bytes)
            Dim readsize As Integer = 1024
            Dim buffer(readsize) As Byte
            Dim count As Integer = stream.Read(buffer, 0, readsize)
            Do While (count > 0)
                For i As Integer = 0 To count - 1
                    ulCRC = (ulCRC Xor ulShiftLeftBy16(CUInt(Bytes(i))))
                    For j As Integer = 0 To 7
                        ulCRC = ulShiftLeftByOne(ulCRC)
                        If (ulCRC And &H1000000) <> 0 Then
                            ulCRC = (ulCRC Xor CRC24_POLY)
                        End If
                    Next
                Next
                count = stream.Read(buffer, 0, readsize)
            Loop
            stream.Close()
            ulCRC = (ulCRC And &HFFFFFF)
            Dim Output As Byte() = BitConverter.GetBytes(CUInt(ulCRC))
            Array.Resize(Output, 3)
            Array.Reverse(Output)
            Return Output
        End Function

        Private Function ulShiftLeftBy16(ByVal wordX As Long) As Long
            ' Shift 32-bit long value to left by 16 bits
            ' i.e. VB equivalent of "wordX << 16" in C
            ' Avoiding problem with sign bit
            ' Copyright (C) 2000-03 DI Management Services Pty Ltd
            ulShiftLeftBy16 = (wordX And &H7FFF&) * &H10000
            If (wordX And &H8000&) <> 0 Then
                ulShiftLeftBy16 = ulShiftLeftBy16 Or &H80000000
            End If
        End Function

        Private Function ulShiftLeftByOne(ByVal wordX As Long) As Long
            ' Shift 32-bit long value to left by 1 bits
            ' i.e. VB equivalent of "wordX << 1" in C
            ' Avoiding problem with sign bit
            ' Copyright (C) 2000-03 DI Management Services Pty Ltd
            ulShiftLeftByOne = (wordX And &H7FFFFFFF) * &H2
            If (wordX And &H8000000) <> 0 Then
                ulShiftLeftByOne = ulShiftLeftByOne Or &H80000000
            End If
        End Function

    End Class

    Public Class CRC_32

        Private CRC_Table(255) As UInteger
        Private CRC_Polynomial As UInteger = &HEDB88320L
        Private CRC_InitialValue As UInteger = &HFFFFFFFFL

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc As UInteger
            For i = 0 To 255
                crc = CUInt(i)
                For j = 0 To 7
                    If (crc And &H1L) Then
                        crc = (crc >> 1) Xor Me.CRC_Polynomial
                    Else
                        crc = (crc >> 1)
                    End If
                Next j
                Me.CRC_Table(i) = crc
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UInteger = Me.CRC_InitialValue
            For Each b As Byte In bytes
                Dim sum1 As UInteger
                Dim sum2 As UInteger
                sum2 = (&HFFL And CUInt(b))
                sum1 = (result Xor sum2)
                result = (result >> 8) Xor Me.CRC_Table(sum1 And &HFF)
            Next
            result = result Xor &HFFFFFFFFL
            bytes = BitConverter.GetBytes(CUInt(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_32_BZIP2

        Private CRC_Table(255) As UInteger
        Private CRC_Polynomial As UInteger = &H4C11DB7L
        Private CRC_InitialValue As UInteger = &HFFFFFFFFL

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc_accum As UInteger
            For i = 0 To 255
                crc_accum = (CUInt(i) << 24)
                For j = 0 To 7
                    If (crc_accum And &H80000000L) Then
                        crc_accum = ((crc_accum << 1) Xor Me.CRC_Polynomial)
                    Else
                        crc_accum = (crc_accum << 1)
                    End If
                Next j
                CRC_Table(i) = crc_accum
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UInteger = Me.CRC_InitialValue
            For Each b As Byte In bytes
                Dim i As Integer
                i = (CInt(Fix(result >> 24)) Xor b) And &HFF
                result = (result << 8) Xor CRC_Table(i)
            Next
            result = (result Xor &HFFFFFFFFL)
            bytes = BitConverter.GetBytes(CUInt(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_32_JAM

        Private CRC_Table(255) As UInteger
        Private CRC_Polynomial As UInteger = &HEDB88320UI
        Private CRC_InitialValue As UInteger = &HFFFFFFFFUI

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc As UInteger
            For i = 0 To 255
                crc = CUInt(i)
                For j = 0 To 7
                    If (crc And 1) <> 0 Then
                        crc = (crc >> 1) Xor Me.CRC_Polynomial
                    Else
                        crc = (crc >> 1)
                    End If
                Next j
                Me.CRC_Table(i) = crc
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim tmp As UInteger = Me.CRC_InitialValue
            For Each b As Byte In bytes
                tmp = (tmp >> 8) Xor Me.CRC_Table(CUInt(tmp Xor b) And &HFF)
            Next
            bytes = BitConverter.GetBytes(CUInt(tmp))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_32_MPEG2

        Private crctab() As UInteger = {&H0, &H4C11DB7, &H9823B6E, &HD4326D9, &H130476DC, &H17C56B6B, &H1A864DB2, &H1E475005, &H2608EDB8, &H22C9F00F, &H2F8AD6D6, &H2B4BCB61, &H350C9B64, &H31CD86D3, &H3C8EA00A, &H384FBDBD, &H4C11DB70, &H48D0C6C7, &H4593E01E, &H4152FDA9, &H5F15ADAC, &H5BD4B01B, &H569796C2, &H52568B75, &H6A1936C8, &H6ED82B7F, &H639B0DA6, &H675A1011, &H791D4014, &H7DDC5DA3, &H709F7B7A, &H745E66CD, &H9823B6E0L, &H9CE2AB57L, &H91A18D8EL, &H95609039L, &H8B27C03CL, &H8FE6DD8BL, &H82A5FB52L, &H8664E6E5L, &HBE2B5B58L, &HBAEA46EFL, &HB7A96036L, &HB3687D81L, &HAD2F2D84L, &HA9EE3033L, &HA4AD16EAL, &HA06C0B5DL, &HD4326D90L, &HD0F37027L, &HDDB056FEL, &HD9714B49L, &HC7361B4CL, &HC3F706FBL, &HCEB42022L, &HCA753D95L, &HF23A8028L, &HF6FB9D9FL, &HFBB8BB46L, &HFF79A6F1L, &HE13EF6F4L, &HE5FFEB43L, &HE8BCCD9AL, &HEC7DD02DL, &H34867077, &H30476DC0, &H3D044B19, &H39C556AE, &H278206AB, &H23431B1C, &H2E003DC5, &H2AC12072, &H128E9DCF, &H164F8078, &H1B0CA6A1, &H1FCDBB16, &H18AEB13, &H54BF6A4, &H808D07D, &HCC9CDCA, &H7897AB07, &H7C56B6B0, &H71159069, &H75D48DDE, &H6B93DDDB, &H6F52C06C, &H6211E6B5, &H66D0FB02, &H5E9F46BF, &H5A5E5B08, &H571D7DD1, &H53DC6066, &H4D9B3063, &H495A2DD4, &H44190B0D, &H40D816BA, &HACA5C697L, &HA864DB20L, &HA527FDF9L, &HA1E6E04EL, &HBFA1B04BL, &HBB60ADFCL, &HB6238B25L, &HB2E29692L, &H8AAD2B2FL, &H8E6C3698L, &H832F1041L, &H87EE0DF6L, &H99A95DF3L, &H9D684044L, &H902B669DL, &H94EA7B2AL, &HE0B41DE7L, &HE4750050L, &HE9362689L, &HEDF73B3EL, &HF3B06B3BL, &HF771768CL, &HFA325055L, &HFEF34DE2L, &HC6BCF05FL, &HC27DEDE8L, &HCF3ECB31L, &HCBFFD686L, &HD5B88683L, &HD1799B34L, &HDC3ABDEDL, &HD8FBA05AL, &H690CE0EE, &H6DCDFD59, &H608EDB80, &H644FC637, &H7A089632, &H7EC98B85, &H738AAD5C, &H774BB0EB, &H4F040D56, &H4BC510E1, &H46863638, &H42472B8F, &H5C007B8A, &H58C1663D, &H558240E4, &H51435D53, &H251D3B9E, &H21DC2629, &H2C9F00F0, &H285E1D47, &H36194D42, &H32D850F5, &H3F9B762C, &H3B5A6B9B, &H315D626, &H7D4CB91, &HA97ED48, &HE56F0FF, &H1011A0FA, &H14D0BD4D, &H19939B94, &H1D528623, &HF12F560EL, &HF5EE4BB9L, &HF8AD6D60L, &HFC6C70D7L, &HE22B20D2L, &HE6EA3D65L, &HEBA91BBCL, &HEF68060BL, &HD727BBB6L, &HD3E6A601L, &HDEA580D8L, &HDA649D6FL, &HC423CD6AL, &HC0E2D0DDL, &HCDA1F604L, &HC960EBB3L, &HBD3E8D7EL, &HB9FF90C9L, &HB4BCB610L, &HB07DABA7L, &HAE3AFBA2L, &HAAFBE615L, &HA7B8C0CCL, &HA379DD7BL, &H9B3660C6L, &H9FF77D71L, &H92B45BA8L, &H9675461FL, &H8832161AL, &H8CF30BADL, &H81B02D74L, &H857130C3L, &H5D8A9099, &H594B8D2E, &H5408ABF7, &H50C9B640, &H4E8EE645, &H4A4FFBF2, &H470CDD2B, &H43CDC09C, &H7B827D21, &H7F436096, &H7200464F, &H76C15BF8, &H68860BFD, &H6C47164A, &H61043093, &H65C52D24, &H119B4BE9, &H155A565E, &H18197087, &H1CD86D30, &H29F3D35, &H65E2082, &HB1D065B, &HFDC1BEC, &H3793A651, &H3352BBE6, &H3E119D3F, &H3AD08088, &H2497D08D, &H2056CD3A, &H2D15EBE3, &H29D4F654, &HC5A92679L, &HC1683BCEL, &HCC2B1D17L, &HC8EA00A0L, &HD6AD50A5L, &HD26C4D12L, &HDF2F6BCBL, &HDBEE767CL, &HE3A1CBC1L, &HE760D676L, &HEA23F0AFL, &HEEE2ED18L, &HF0A5BD1DL, &HF464A0AAL, &HF9278673L, &HFDE69BC4L, &H89B8FD09L, &H8D79E0BEL, &H803AC667L, &H84FBDBD0L, &H9ABC8BD5L, &H9E7D9662L, &H933EB0BBL, &H97FFAD0CL, &HAFB010B1L, &HAB710D06L, &HA6322BDFL, &HA2F33668L, &HBCB4666DL, &HB8757BDAL, &HB5365D03L, &HB1F740B4L}

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim tmp As UInteger = &HFFFFFFFFUI
            For Each b As Byte In bytes
                tmp = (tmp << 8) Xor crctab(((tmp >> 24) Xor b) And &HFF)
            Next
            bytes = BitConverter.GetBytes(CUInt(tmp))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class CRC_64_ECMA

        Private CRC_Polynomial As ULong = &HC96C5795D7870F42UL
        Private CRC_InitialValue As ULong = &HFFFFFFFFFFFFFFFFUL

#Region "CRC_Table"
        Private CRC_Table() As ULong = {&H0, &H42F0E1EBA9EA3693L, &H85E1C3D753D46D26UL, &HC711223CFA3E5BB5UL, &H493366450E42ECDFL, &HBC387AEA7A8DA4CL, _
        &HCCD2A5925D9681F9UL, &H8E224479F47CB76AUL, &H9266CC8A1C85D9BEUL, &HD0962D61B56FEF2DUL, &H17870F5D4F51B498L, &H5577EEB6E6BB820BL, _
        &HDB55AACF12C73561UL, &H99A54B24BB2D03F2UL, &H5EB4691841135847L, &H1C4488F3E8F96ED4L, &H663D78FF90E185EFL, &H24CD9914390BB37CL, _
        &HE3DCBB28C335E8C9UL, &HA12C5AC36ADFDE5AUL, &H2F0E1EBA9EA36930L, &H6DFEFF5137495FA3L, &HAAEFDD6DCD770416UL, &HE81F3C86649D3285UL, _
        &HF45BB4758C645C51UL, &HB6AB559E258E6AC2UL, &H71BA77A2DFB03177L, &H334A9649765A07E4L, &HBD68D2308226B08EUL, &HFF9833DB2BCC861DUL, _
        &H388911E7D1F2DDA8L, &H7A79F00C7818EB3BL, &HCC7AF1FF21C30BDEUL, &H8E8A101488293D4DUL, &H499B3228721766F8L, &HB6BD3C3DBFD506BL, _
        &H854997BA2F81E701UL, &HC7B97651866BD192UL, &HA8546D7C558A27L, &H4258B586D5BFBCB4L, &H5E1C3D753D46D260L, &H1CECDC9E94ACE4F3L, _
        &HDBFDFEA26E92BF46UL, &H990D1F49C77889D5UL, &H172F5B3033043EBFL, &H55DFBADB9AEE082CL, &H92CE98E760D05399UL, &HD03E790CC93A650AUL, _
        &HAA478900B1228E31UL, &HE8B768EB18C8B8A2UL, &H2FA64AD7E2F6E317L, &H6D56AB3C4B1CD584L, &HE374EF45BF6062EEUL, &HA1840EAE168A547DUL, _
        &H66952C92ECB40FC8L, &H2465CD79455E395BL, &H3821458AADA7578FL, &H7AD1A461044D611CL, &HBDC0865DFE733AA9UL, &HFF3067B657990C3AUL, _
        &H711223CFA3E5BB50L, &H33E2C2240A0F8DC3L, &HF4F3E018F031D676UL, &HB60301F359DBE0E5UL, &HDA050215EA6C212FUL, &H98F5E3FE438617BCUL, _
        &H5FE4C1C2B9B84C09L, &H1D14202910527A9AL, &H93366450E42ECDF0UL, &HD1C685BB4DC4FB63UL, &H16D7A787B7FAA0D6L, &H5427466C1E109645L, _
        &H4863CE9FF6E9F891L, &HA932F745F03CE02L, &HCD820D48A53D95B7UL, &H8F72ECA30CD7A324UL, &H150A8DAF8AB144EL, &H43A04931514122DDL, _
        &H84B16B0DAB7F7968UL, &HC6418AE602954FFBUL, &HBC387AEA7A8DA4C0UL, &HFEC89B01D3679253UL, &H39D9B93D2959C9E6L, &H7B2958D680B3FF75L, _
        &HF50B1CAF74CF481FUL, &HB7FBFD44DD257E8CUL, &H70EADF78271B2539L, &H321A3E938EF113AAL, &H2E5EB66066087D7EL, &H6CAE578BCFE24BEDL, _
        &HABBF75B735DC1058UL, &HE94F945C9C3626CBUL, &H676DD025684A91A1L, &H259D31CEC1A0A732L, &HE28C13F23B9EFC87UL, &HA07CF2199274CA14UL, _
        &H167FF3EACBAF2AF1L, &H548F120162451C62L, &H939E303D987B47D7UL, &HD16ED1D631917144UL, &H5F4C95AFC5EDC62EL, &H1DBC74446C07F0BDL, _
        &HDAAD56789639AB08UL, &H985DB7933FD39D9BUL, &H84193F60D72AF34FUL, &HC6E9DE8B7EC0C5DCUL, &H1F8FCB784FE9E69L, &H43081D5C2D14A8FAL, _
        &HCD2A5925D9681F90UL, &H8FDAB8CE70822903UL, &H48CB9AF28ABC72B6L, &HA3B7B1923564425L, &H70428B155B4EAF1EL, &H32B26AFEF2A4998DL, _
        &HF5A348C2089AC238UL, &HB753A929A170F4ABUL, &H3971ED50550C43C1L, &H7B810CBBFCE67552L, &HBC902E8706D82EE7UL, &HFE60CF6CAF321874UL, _
        &HE224479F47CB76A0UL, &HA0D4A674EE214033UL, &H67C58448141F1B86L, &H253565A3BDF52D15L, &HAB1721DA49899A7FUL, &HE9E7C031E063ACECUL, _
        &H2EF6E20D1A5DF759L, &H6C0603E6B3B7C1CAL, &HF6FAE5C07D3274CDUL, &HB40A042BD4D8425EUL, &H731B26172EE619EBL, &H31EBC7FC870C2F78L, _
        &HBFC9838573709812UL, &HFD39626EDA9AAE81UL, &H3A28405220A4F534L, &H78D8A1B9894EC3A7L, &H649C294A61B7AD73L, &H266CC8A1C85D9BE0L, _
        &HE17DEA9D3263C055UL, &HA38D0B769B89F6C6UL, &H2DAF4F0F6FF541ACL, &H6F5FAEE4C61F773FL, &HA84E8CD83C212C8AUL, &HEABE6D3395CB1A19UL, _
        &H90C79D3FEDD3F122UL, &HD2377CD44439C7B1UL, &H15265EE8BE079C04L, &H57D6BF0317EDAA97L, &HD9F4FB7AE3911DFDUL, &H9B041A914A7B2B6EUL, _
        &H5C1538ADB04570DBL, &H1EE5D94619AF4648L, &H2A151B5F156289CL, &H4051B05E58BC1E0FL, &H87409262A28245BAUL, &HC5B073890B687329UL, _
        &H4B9237F0FF14C443L, &H962D61B56FEF2D0L, &HCE73F427ACC0A965UL, &H8C8315CC052A9FF6UL, &H3A80143F5CF17F13L, &H7870F5D4F51B4980L, _
        &HBF61D7E80F251235UL, &HFD913603A6CF24A6UL, &H73B3727A52B393CCL, &H31439391FB59A55FL, &HF652B1AD0167FEEAUL, &HB4A25046A88DC879UL, _
        &HA8E6D8B54074A6ADUL, &HEA16395EE99E903EUL, &H2D071B6213A0CB8BL, &H6FF7FA89BA4AFD18L, &HE1D5BEF04E364A72UL, &HA3255F1BE7DC7CE1UL, _
        &H64347D271DE22754L, &H26C49CCCB40811C7L, &H5CBD6CC0CC10FAFCL, &H1E4D8D2B65FACC6FL, &HD95CAF179FC497DAUL, &H9BAC4EFC362EA149UL, _
        &H158E0A85C2521623L, &H577EEB6E6BB820B0L, &H906FC95291867B05UL, &HD29F28B9386C4D96UL, &HCEDBA04AD0952342UL, &H8C2B41A1797F15D1UL, _
        &H4B3A639D83414E64L, &H9CA82762AAB78F7L, &H87E8C60FDED7CF9DUL, &HC51827E4773DF90EUL, &H20905D88D03A2BBL, &H40F9E43324E99428L, _
        &H2CFFE7D5975E55E2L, &H6E0F063E3EB46371L, &HA91E2402C48A38C4UL, &HEBEEC5E96D600E57UL, &H65CC8190991CB93DL, &H273C607B30F68FAEL, _
        &HE02D4247CAC8D41BUL, &HA2DDA3AC6322E288UL, &HBE992B5F8BDB8C5CUL, &HFC69CAB42231BACFUL, &H3B78E888D80FE17AL, &H7988096371E5D7E9L, _
        &HF7AA4D1A85996083UL, &HB55AACF12C735610UL, &H724B8ECDD64D0DA5L, &H30BB6F267FA73B36L, &H4AC29F2A07BFD00DL, &H8327EC1AE55E69EL, _
        &HCF235CFD546BBD2BUL, &H8DD3BD16FD818BB8UL, &H3F1F96F09FD3CD2L, &H41011884A0170A41L, &H86103AB85A2951F4UL, &HC4E0DB53F3C36767UL, _
        &HD8A453A01B3A09B3UL, &H9A54B24BB2D03F20UL, &H5D45907748EE6495L, &H1FB5719CE1045206L, &H919735E51578E56CUL, &HD367D40EBC92D3FFUL, _
        &H1476F63246AC884AL, &H568617D9EF46BED9L, &HE085162AB69D5E3CUL, &HA275F7C11F7768AFUL, &H6564D5FDE549331AL, &H279434164CA30589L, _
        &HA9B6706FB8DFB2E3UL, &HEB46918411358470UL, &H2C57B3B8EB0BDFC5L, &H6EA7525342E1E956L, &H72E3DAA0AA188782L, &H30133B4B03F2B111L, _
        &HF7021977F9CCEAA4UL, &HB5F2F89C5026DC37UL, &H3BD0BCE5A45A6B5DL, &H79205D0E0DB05DCEL, &HBE317F32F78E067BUL, &HFCC19ED95E6430E8UL, _
        &H86B86ED5267CDBD3UL, &HC4488F3E8F96ED40UL, &H359AD0275A8B6F5L, &H41A94CE9DC428066L, &HCF8B0890283E370CUL, &H8D7BE97B81D4019FUL, _
        &H4A6ACB477BEA5A2AL, &H89A2AACD2006CB9L, &H14DEA25F3AF9026DL, &H562E43B4931334FEL, &H913F6188692D6F4BUL, &HD3CF8063C0C759D8UL, _
        &H5DEDC41A34BBEEB2L, &H1F1D25F19D51D821L, &HD80C07CD676F8394UL, &H9AFCE626CE85B507UL}
#End Region

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As ULong = Me.CRC_InitialValue 'Initial
            For Each b As Byte In bytes
                result = CRC_Table(CByte(result >> 56) Xor b) Xor (result << 8)
            Next
            result = (result Xor &HFFFFFFFFFFFFFFFFUL)
            bytes = BitConverter.GetBytes(CULng(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class DEK_HASH

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim hash As UInteger = bytes.Length
            For Each b As Byte In bytes
                hash = ((hash << 5) Xor (hash >> 27)) Xor (b)
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class DJB_HASH

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim hash As UInteger = 5381
            Dim i As UInteger = 0
            For Each b As Byte In bytes
                hash = ((hash << 5) + hash) + (b)
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class ELF_HASH

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim hash As UInteger = 0
            Dim x As UInteger = 0
            For Each b As Byte In bytes
                hash = (hash << 4) + (b)
                x = hash And &HF0000000L
                If x <> 0 Then
                    hash = hash Xor (x >> 24)
                End If
                hash = hash And Not x
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class FCS_16

        Private FCSINIT16 As UShort = &HFFFFUS
        Private fcs16tab() As UShort = {&H0, &H1189, &H2312, &H329B, &H4624, &H57AD, &H6536, &H74BF, &H8C48, &H9DC1, &HAF5A, &HBED3, &HCA6C, &HDBE5, &HE97E, &HF8F7, &H1081, &H108, &H3393, &H221A, &H56A5, &H472C, &H75B7, &H643E, &H9CC9, &H8D40, &HBFDB, &HAE52, &HDAED, &HCB64, &HF9FF, &HE876, &H2102, &H308B, &H210, &H1399, &H6726, &H76AF, &H4434, &H55BD, &HAD4A, &HBCC3, &H8E58, &H9FD1, &HEB6E, &HFAE7, &HC87C, &HD9F5, &H3183, &H200A, &H1291, &H318, &H77A7, &H662E, &H54B5, &H453C, &HBDCB, &HAC42, &H9ED9, &H8F50, &HFBEF, &HEA66, &HD8FD, &HC974, &H4204, &H538D, &H6116, &H709F, &H420, &H15A9, &H2732, &H36BB, &HCE4C, &HDFC5, &HED5E, &HFCD7, &H8868, &H99E1, &HAB7A, &HBAF3, &H5285, &H430C, &H7197, &H601E, &H14A1, &H528, &H37B3, &H263A, &HDECD, &HCF44, &HFDDF, &HEC56, &H98E9, &H8960, &HBBFB, &HAA72, &H6306, &H728F, &H4014, &H519D, &H2522, &H34AB, &H630, &H17B9, &HEF4E, &HFEC7, &HCC5C, &HDDD5, &HA96A, &HB8E3, &H8A78, &H9BF1, &H7387, &H620E, &H5095, &H411C, &H35A3, &H242A, &H16B1, &H738, &HFFCF, &HEE46, &HDCDD, &HCD54, &HB9EB, &HA862, &H9AF9, &H8B70, &H8408, &H9581, &HA71A, &HB693, &HC22C, &HD3A5, &HE13E, &HF0B7, &H840, &H19C9, &H2B52, &H3ADB, &H4E64, &H5FED, &H6D76, &H7CFF, &H9489, &H8500, &HB79B, &HA612, &HD2AD, &HC324, &HF1BF, &HE036, &H18C1, &H948, &H3BD3, &H2A5A, &H5EE5, &H4F6C, &H7DF7, &H6C7E, &HA50A, &HB483, &H8618, &H9791, &HE32E, &HF2A7, &HC03C, &HD1B5, &H2942, &H38CB, &HA50, &H1BD9, &H6F66, &H7EEF, &H4C74, &H5DFD, &HB58B, &HA402, &H9699, &H8710, &HF3AF, &HE226, &HD0BD, &HC134, &H39C3, &H284A, &H1AD1, &HB58, &H7FE7, &H6E6E, &H5CF5, &H4D7C, &HC60C, &HD785, &HE51E, &HF497, &H8028, &H91A1, &HA33A, &HB2B3, &H4A44, &H5BCD, &H6956, &H78DF, &HC60, &H1DE9, &H2F72, &H3EFB, &HD68D, &HC704, &HF59F, &HE416, &H90A9, &H8120, &HB3BB, &HA232, &H5AC5, &H4B4C, &H79D7, &H685E, &H1CE1, &HD68, &H3FF3, &H2E7A, &HE70E, &HF687, &HC41C, &HD595, &HA12A, &HB0A3, &H8238, &H93B1, &H6B46, &H7ACF, &H4854, &H59DD, &H2D62, &H3CEB, &HE70, &H1FF9, &HF78F, &HE606, &HD49D, &HC514, &HB1AB, &HA022, &H92B9, &H8330, &H7BC7, &H6A4E, &H58D5, &H495C, &H3DE3, &H2C6A, &H1EF1, &HF78}

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UShort = Me.FCSINIT16
            For Each b As Byte In bytes
                result = (result >> 8) Xor Me.fcs16tab((result Xor b) And &HFF)
            Next
            result = Not result
            bytes = BitConverter.GetBytes(CUShort(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class FCS_32

        Private FCSINIT32 As UInteger = &HFFFFFFFFUI
        Private fcs32tab() As UInteger = {&H0, &H77073096, &HEE0E612CL, &H990951BAL, &H76DC419, &H706AF48F, &HE963A535L, &H9E6495A3L, &HEDB8832, &H79DCB8A4, &HE0D5E91EL, &H97D2D988L, &H9B64C2B, &H7EB17CBD, &HE7B82D07L, &H90BF1D91L, &H1DB71064, &H6AB020F2, &HF3B97148L, &H84BE41DEL, &H1ADAD47D, &H6DDDE4EB, &HF4D4B551L, &H83D385C7L, &H136C9856, &H646BA8C0, &HFD62F97AL, &H8A65C9ECL, &H14015C4F, &H63066CD9, &HFA0F3D63L, &H8D080DF5L, &H3B6E20C8, &H4C69105E, &HD56041E4L, &HA2677172L, &H3C03E4D1, &H4B04D447, &HD20D85FDL, &HA50AB56BL, &H35B5A8FA, &H42B2986C, &HDBBBC9D6L, &HACBCF940L, &H32D86CE3, &H45DF5C75, &HDCD60DCFL, &HABD13D59L, &H26D930AC, &H51DE003A, &HC8D75180L, &HBFD06116L, &H21B4F4B5, &H56B3C423, &HCFBA9599L, &HB8BDA50FL, &H2802B89E, &H5F058808, &HC60CD9B2L, &HB10BE924L, &H2F6F7C87, &H58684C11, &HC1611DABL, &HB6662D3DL, &H76DC4190, &H1DB7106, &H98D220BCL, &HEFD5102AL, &H71B18589, &H6B6B51F, &H9FBFE4A5L, &HE8B8D433L, &H7807C9A2, &HF00F934, &H9609A88EL, &HE10E9818L, &H7F6A0DBB, &H86D3D2D, &H91646C97L, &HE6635C01L, &H6B6B51F4, &H1C6C6162, &H856530D8L, &HF262004EL, &H6C0695ED, &H1B01A57B, &H8208F4C1L, &HF50FC457L, &H65B0D9C6, &H12B7E950, &H8BBEB8EAL, &HFCB9887CL, &H62DD1DDF, &H15DA2D49, &H8CD37CF3L, &HFBD44C65L, &H4DB26158, &H3AB551CE, &HA3BC0074L, &HD4BB30E2L, &H4ADFA541, &H3DD895D7, &HA4D1C46DL, &HD3D6F4FBL, &H4369E96A, &H346ED9FC, &HAD678846L, &HDA60B8D0L, &H44042D73, &H33031DE5, &HAA0A4C5FL, &HDD0D7CC9L, &H5005713C, &H270241AA, &HBE0B1010L, &HC90C2086L, &H5768B525, &H206F85B3, &HB966D409L, &HCE61E49FL, &H5EDEF90E, &H29D9C998, &HB0D09822L, &HC7D7A8B4L, &H59B33D17, &H2EB40D81, &HB7BD5C3BL, &HC0BA6CADL, &HEDB88320L, &H9ABFB3B6L, &H3B6E20C, &H74B1D29A, &HEAD54739L, &H9DD277AFL, &H4DB2615, &H73DC1683, &HE3630B12L, &H94643B84L, &HD6D6A3E, &H7A6A5AA8, &HE40ECF0BL, &H9309FF9DL, &HA00AE27, &H7D079EB1, &HF00F9344L, &H8708A3D2L, &H1E01F268, &H6906C2FE, &HF762575DL, &H806567CBL, &H196C3671, &H6E6B06E7, &HFED41B76L, &H89D32BE0L, &H10DA7A5A, &H67DD4ACC, &HF9B9DF6FL, &H8EBEEFF9L, &H17B7BE43, &H60B08ED5, &HD6D6A3E8L, &HA1D1937EL, &H38D8C2C4, &H4FDFF252, &HD1BB67F1L, &HA6BC5767L, &H3FB506DD, &H48B2364B, &HD80D2BDAL, &HAF0A1B4CL, &H36034AF6, &H41047A60, &HDF60EFC3L, &HA867DF55L, &H316E8EEF, &H4669BE79, &HCB61B38CL, &HBC66831AL, &H256FD2A0, &H5268E236, &HCC0C7795L, &HBB0B4703L, &H220216B9, &H5505262F, &HC5BA3BBEL, &HB2BD0B28L, &H2BB45A92, &H5CB36A04, &HC2D7FFA7L, &HB5D0CF31L, &H2CD99E8B, &H5BDEAE1D, &H9B64C2B0L, &HEC63F226L, &H756AA39C, &H26D930A, &H9C0906A9L, &HEB0E363FL, &H72076785, &H5005713, &H95BF4A82L, &HE2B87A14L, &H7BB12BAE, &HCB61B38, &H92D28E9BL, &HE5D5BE0DL, &H7CDCEFB7, &HBDBDF21, &H86D3D2D4L, &HF1D4E242L, &H68DDB3F8, &H1FDA836E, &H81BE16CDL, &HF6B9265BL, &H6FB077E1, &H18B74777, &H88085AE6L, &HFF0F6A70L, &H66063BCA, &H11010B5C, &H8F659EFFL, &HF862AE69L, &H616BFFD3, &H166CCF45, &HA00AE278L, &HD70DD2EEL, &H4E048354, &H3903B3C2, &HA7672661L, &HD06016F7L, &H4969474D, &H3E6E77DB, &HAED16A4AL, &HD9D65ADCL, &H40DF0B66, &H37D83BF0, &HA9BCAE53L, &HDEBB9EC5L, &H47B2CF7F, &H30B5FFE9, &HBDBDF21CL, &HCABAC28AL, &H53B39330, &H24B4A3A6, &HBAD03605L, &HCDD70693L, &H54DE5729, &H23D967BF, &HB3667A2EL, &HC4614AB8L, &H5D681B02, &H2A6F2B94, &HB40BBE37L, &HC30C8EA1L, &H5A05DF1B, &H2D02EF8D}

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UInteger = Me.FCSINIT32
            For Each b As Byte In bytes
                result = (result >> 8) Xor Me.fcs32tab((result Xor b) And &HFF)
            Next
            result = Not result
            bytes = BitConverter.GetBytes(CUInt(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class FLETCHER_8

        Public Function Compute(ByVal bytes As Byte()) As Byte
            Dim sum1 As Byte = &HF
            Dim sum2 As Byte = &HF
            For Each b As Byte In bytes
                sum1 = (sum1 + b)
                sum2 = (sum2 + sum1)
                sum1 = (sum1 And &HF) + (sum1 >> 4)
                sum2 = (sum2 And &HF) + (sum2 >> 4)
            Next
            sum1 = (sum1 And &HF) + (sum1 >> 4)
            sum2 = (sum2 And &HF) + (sum2 >> 4)
            Return CByte((sum2 << 4) Or sum1)
        End Function

    End Class

    Public Class FLETCHER_16

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim sum1 As UShort = &HFF
            Dim sum2 As UShort = &HFF
            For Each b As Byte In bytes
                sum1 = (sum1 + b)
                sum2 = (sum2 + sum1)
                sum1 = (sum1 And &HFF) + (sum1 >> 8)
                sum2 = (sum2 And &HFF) + (sum2 >> 8)
            Next
            sum1 = (sum1 And &HFF) + (sum1 >> 8)
            sum2 = (sum2 And &HFF) + (sum2 >> 8)
            bytes = BitConverter.GetBytes(CUShort((sum2 << 8) Or sum1))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class FLETCHER_32

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim sum1 As UInteger = &HFFFF
            Dim sum2 As UInteger = &HFFFF
            For Each b As Byte In bytes
                sum1 = (sum1 + b)
                sum2 = (sum2 + sum1)
                sum1 = (sum1 And &HFFFF) + (sum1 >> 16)
                sum2 = (sum2 And &HFFFF) + (sum2 >> 16)
            Next
            sum1 = (sum1 And &HFFFF) + (sum1 >> 16)
            sum2 = (sum2 And &HFFFF) + (sum2 >> 16)
            bytes = BitConverter.GetBytes(CUInt((sum2 << 16) Or sum1))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class FNV0_32

        Private FNV_PRIME As UInteger = 16777619
        Private FNV_OFFSET_BASIS As UInteger = 0

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UInteger = Me.FNV_OFFSET_BASIS
            For Each b As Byte In bytes
                result = (result * Me.FNV_PRIME) And &HFFFFFFFFUI
                result = (result Xor b)
            Next
            bytes = BitConverter.GetBytes(CUInt(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class FNV1_32

        Private FNV_PRIME As UInteger = 16777619
        Private FNV_OFFSET_BASIS As UInteger = 2166136261UI

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UInteger = Me.FNV_OFFSET_BASIS
            For Each b As Byte In bytes
                result = (result * Me.FNV_PRIME) And &HFFFFFFFFUI
                result = (result Xor b)
            Next
            bytes = BitConverter.GetBytes(CUInt(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class FNV1a_32

        Private FNV_PRIME As UInteger = 16777619
        Private FNV_OFFSET_BASIS As UInteger = 2166136261UI

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UInteger = Me.FNV_OFFSET_BASIS
            For Each b As Byte In bytes
                result = (result Xor b)
                result = (result * Me.FNV_PRIME) And &HFFFFFFFFUI
            Next
            bytes = BitConverter.GetBytes(CUInt(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class GHASH_3

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim value As UInteger = 0
            Dim tmp As UInteger
            For Each b As Byte In bytes
                tmp = value
                value = (tmp << 3) + tmp + b
            Next
            bytes = BitConverter.GetBytes(CUInt(value))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class GHASH_5

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim value As UInteger = 0
            Dim tmp As UInteger
            For Each b As Byte In bytes
                tmp = value
                value = (tmp << 5) + tmp + b
            Next
            bytes = BitConverter.GetBytes(CUInt(value))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class HMAC_MD_5

        Private Hash As New System.Security.Cryptography.HMACMD5

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class HMAC_RIPE_MD_160

        Private Hash As New System.Security.Cryptography.HMACRIPEMD160

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class HMAC_SHA_1

        Private Hash As New System.Security.Cryptography.HMACSHA1

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class HMAC_SHA_256

        Private Hash As New System.Security.Cryptography.HMACSHA256

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class HMAC_SHA_384

        Private Hash As New System.Security.Cryptography.HMACSHA384

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class HMAC_SHA_512

        Private Hash As New System.Security.Cryptography.HMACSHA512

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class JS_HASH

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim hash As UInteger = 1315423911
            For Each bit As Byte In bytes
                hash = (hash Xor ((hash << 5) + (bit) + (hash >> 2)))
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class MAC_TripleDES

        Private Hash As New System.Security.Cryptography.MACTripleDES

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class MD_5

        Private Hash As New System.Security.Cryptography.MD5CryptoServiceProvider

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class MD_5_Cng

        Private Hash As New System.Security.Cryptography.MD5Cng

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class MURMUR_HASH_1

        Public Function Compute(ByVal bytes As Byte(), Optional ByVal seed As UInteger = 0) As Byte()
            Dim len As Integer = bytes.Length
            Dim m As UInteger = &HC6A4A793L
            Dim r As Integer = 16
            Dim h As UInteger = seed Xor (len * m)
            '----------
            Dim i As Integer = 0
            Do While len >= 4
                Dim k As Integer = bytes(i + 0) And &HFF
                k = k Or (bytes(i + 1) And &HFF) << 8
                k = k Or (bytes(i + 2) And &HFF) << 16
                k = k Or (bytes(i + 3) And &HFF) << 24

                h += k
                h *= m
                h = h Xor h >> 16
                i += 4
                len -= 4
            Loop
            '----------
            Select Case len
                Case 3
                    h += bytes(i + 2) << 16
                Case 2
                    h += bytes(i + 1) << 8
                Case 1
                    h += bytes(i + 0)
                    h *= m
                    h = h Xor h >> r
            End Select
            '----------
            h *= m
            h = h Xor h >> 10
            h *= m
            h = h Xor h >> 17
            '----------
            bytes = BitConverter.GetBytes(CUInt(h))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class MURMUR_HASH_1_FFFFFFFF

        Public Function Compute(ByVal bytes As Byte(), Optional ByVal seed As UInteger = &HFFFFFFFFUI) As Byte()
            Dim len As Integer = bytes.Length
            Dim m As UInteger = &HC6A4A793L
            Dim r As Integer = 16
            Dim h As UInteger = seed Xor (len * m)
            '----------
            Dim i As Integer = 0
            Do While len >= 4
                Dim k As Integer = bytes(i + 0) And &HFF
                k = k Or (bytes(i + 1) And &HFF) << 8
                k = k Or (bytes(i + 2) And &HFF) << 16
                k = k Or (bytes(i + 3) And &HFF) << 24

                h += k
                h *= m
                h = h Xor h >> 16
                i += 4
                len -= 4
            Loop
            '----------
            Select Case len
                Case 3
                    h += bytes(i + 2) << 16
                Case 2
                    h += bytes(i + 1) << 8
                Case 1
                    h += bytes(i + 0)
                    h *= m
                    h = h Xor h >> r
            End Select
            '----------
            h *= m
            h = h Xor h >> 10
            h *= m
            h = h Xor h >> 17
            '----------
            bytes = BitConverter.GetBytes(CUInt(h))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class MURMUR_HASH_2

        Public Function Compute(ByVal bytes As Byte(), Optional ByVal seed As UInteger = 0) As Byte()
            ' 'm' and 'r' are mixing constants generated offline.
            ' They're not really 'magic', they just happen to work well.
            Dim m As UInteger = &H5BD1E995
            Dim r As UInteger = 24

            ' Initialize the hash to a 'random' value
            Dim len As UInteger = bytes.Length
            Dim h As UInteger = seed Xor len

            Dim i As UInteger = 0
            Do While len >= 4
                Dim k As Integer = bytes(i + 0) And &HFF
                k = k Or (bytes(i + 1) And &HFF) << 8
                k = k Or (bytes(i + 2) And &HFF) << 16
                k = k Or (bytes(i + 3) And &HFF) << 24

                k *= m
                k = k Xor k >> r
                k *= m

                h *= m
                h = h Xor k

                i += 4
                len -= 4
            Loop

            Select Case len
                Case 3
                    h = h Xor (bytes(i + 2) And &HFF) << 16
                Case 2
                    h = h Xor (bytes(i + 1) And &HFF) << 8
                Case 1
                    h = h Xor (bytes(i + 0) And &HFF)
                    h *= m
            End Select

            h = h Xor h >> 13
            h *= m
            h = h Xor h >> 15

            bytes = BitConverter.GetBytes(CUInt(h))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class MURMUR_HASH_2_FFFFFFFF

        Public Function Compute(ByVal bytes As Byte(), Optional ByVal seed As UInteger = &HFFFFFFFFUI) As Byte()
            ' 'm' and 'r' are mixing constants generated offline.
            ' They're not really 'magic', they just happen to work well.
            Dim m As UInteger = &H5BD1E995
            Dim r As UInteger = 24

            ' Initialize the hash to a 'random' value
            Dim len As UInteger = bytes.Length
            Dim h As UInteger = seed Xor len

            Dim i As UInteger = 0
            Do While len >= 4
                Dim k As Integer = bytes(i + 0) And &HFF
                k = k Or (bytes(i + 1) And &HFF) << 8
                k = k Or (bytes(i + 2) And &HFF) << 16
                k = k Or (bytes(i + 3) And &HFF) << 24

                k *= m
                k = k Xor k >> r
                k *= m

                h *= m
                h = h Xor k

                i += 4
                len -= 4
            Loop

            Select Case len
                Case 3
                    h = h Xor (bytes(i + 2) And &HFF) << 16
                Case 2
                    h = h Xor (bytes(i + 1) And &HFF) << 8
                Case 1
                    h = h Xor (bytes(i + 0) And &HFF)
                    h *= m
            End Select

            h = h Xor h >> 13
            h *= m
            h = h Xor h >> 15

            bytes = BitConverter.GetBytes(CUInt(h))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class ONE_AT_A_TIME_HASH

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim hash As UInteger = 0
            For Each b As Byte In bytes
                hash += b
                hash += (hash << 10)
                hash = hash Xor (hash >> 6)
            Next
            hash += (hash << 3)
            hash = hash Xor (hash >> 11)
            hash += (hash << 15)
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class PJW_HASH_BOOK

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim BitsInUnsignedInt As UInteger = CUInt(System.Runtime.InteropServices.Marshal.SizeOf(UInt32.MaxValue)) * 8
            Dim ThreeQuarters As UInteger = CUInt((BitsInUnsignedInt * 3) / 4)
            Dim OneEighth As UInteger = CUInt(BitsInUnsignedInt \ 8)
            Dim HighBits As UInteger = CUInt(&HFFFFFFFFL) << (BitsInUnsignedInt - OneEighth)
            Dim hash As UInteger = 0
            Dim test As UInteger = 0
            For Each b As Byte In bytes
                hash = (hash << OneEighth) + (b)
                test = hash And HighBits
                If test <> 0 Then
                    hash = ((hash Xor (test >> ThreeQuarters)) And ((Not HighBits)))
                End If
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class RIPE_MD_160_Managed

        Private Hash As New System.Security.Cryptography.RIPEMD160Managed

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class RS_HASH

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim b As UInteger = 378551
            Dim a As UInteger = 63689
            Dim hash As UInteger = 0
            For Each bit As Byte In bytes
                hash = hash * a + (bit)
                a = a * b
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class SDBM_HASH

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim hash As UInteger = 0
            Dim i As UInteger = 0
            For Each b As Byte In bytes
                hash = (b) + (hash << 6) + (hash << 16) - hash
            Next
            bytes = BitConverter.GetBytes(CUInt(hash))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class SHA_1

        Private Hash As New System.Security.Cryptography.SHA1CryptoServiceProvider

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_1_Cng

        Private Hash As New System.Security.Cryptography.SHA1Cng

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_256

        Private Hash As New System.Security.Cryptography.SHA256CryptoServiceProvider

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_256_Cng

        Private Hash As New System.Security.Cryptography.SHA256Cng

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_256_Managed

        Private Hash As New System.Security.Cryptography.SHA256Managed

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_384

        Private Hash As New System.Security.Cryptography.SHA384CryptoServiceProvider

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_384_Cng

        Private Hash As New System.Security.Cryptography.SHA384Cng

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_384_Managed

        Private Hash As New System.Security.Cryptography.SHA384Managed

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_512

        Private Hash As New System.Security.Cryptography.SHA512CryptoServiceProvider

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_512_Cng

        Private Hash As New System.Security.Cryptography.SHA512Cng

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SHA_512_Managed

        Private Hash As New System.Security.Cryptography.SHA512Managed

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Return Hash.ComputeHash(Bytes)
        End Function

    End Class

    Public Class SIZE_HASH_8

        Public Function Compute(ByVal bytes As Byte()) As Byte
            Dim result As Byte = 0
            result = (result + bytes.Length) And &HFF
            Return (CByte(result))
        End Function

    End Class

    Public Class SIZE_HASH_16

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UShort = 0
            result = (result + bytes.Length) And &HFFFF
            bytes = BitConverter.GetBytes(CUShort(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class SIZE_HASH_32

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As UInteger = 0
            result = (result + bytes.Length) And &HFFFFFFFF
            bytes = BitConverter.GetBytes(CUInt(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class SIZE_HASH_64

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim result As ULong = 0
            result = (result + bytes.Length) And &HFFFFFFFFFFFFFFFF
            bytes = BitConverter.GetBytes(CULng(result))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class SUM_BSD

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim value As UInteger = 0
            For Each b As Byte In bytes
                Dim tmp As UInteger = value
                tmp = (tmp >> 1) + ((tmp And 1) << 15)
                tmp += b
                tmp = tmp And &HFFFF
                value = tmp
            Next
            bytes = BitConverter.GetBytes(CUShort(value))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class SUM_SYSV

        Public Function Compute(ByVal Bytes As Byte()) As Byte()
            Dim num As Object = 0
            For Each b As Byte In Bytes
                num = (num + b) And &HFFFF
            Next
            Bytes = BitConverter.GetBytes(CUShort(num))
            Array.Resize(Bytes, 2)
            Array.Reverse(Bytes)
            Return Bytes
        End Function

    End Class

    Public Class SUPER_FAST_HASH

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim dataLength As Int32 = bytes.Length
            If dataLength = 0 Then
                Return New Byte() {CUInt(0)}
            End If
            Dim hash__1 As UInt32 = Convert.ToUInt32(dataLength)
            Dim remainingBytes As Int32 = dataLength And 3
            ' mod 4
            Dim numberOfLoops As Int32 = dataLength >> 2
            ' div 4
            Dim currentIndex As Int32 = 0
            While numberOfLoops > 0
                hash__1 += BitConverter.ToUInt16(bytes, currentIndex)
                Dim tmp As UInt32 = CUInt(BitConverter.ToUInt16(bytes, currentIndex + 2) << 11) Xor hash__1
                hash__1 = (hash__1 << 16) Xor tmp
                hash__1 += hash__1 >> 11
                currentIndex += 4
                numberOfLoops -= 1
            End While

            Select Case remainingBytes
                Case 3
                    hash__1 += BitConverter.ToUInt16(bytes, currentIndex)
                    hash__1 = hash__1 Xor hash__1 << 16
                    hash__1 = hash__1 Xor CUInt(bytes(currentIndex + 2)) << 18
                    hash__1 += hash__1 >> 11
                    Exit Select
                Case 2
                    hash__1 += BitConverter.ToUInt16(bytes, currentIndex)
                    hash__1 = hash__1 Xor hash__1 << 11
                    hash__1 += hash__1 >> 17
                    Exit Select
                Case 1
                    hash__1 += bytes(currentIndex)
                    hash__1 = hash__1 Xor hash__1 << 10
                    hash__1 += hash__1 >> 1
                    Exit Select
                Case Else
                    Exit Select
            End Select

            ' Force "avalanching" of final 127 bits 

            hash__1 = hash__1 Xor hash__1 << 3
            hash__1 += hash__1 >> 5
            hash__1 = hash__1 Xor hash__1 << 4
            hash__1 += hash__1 >> 17
            hash__1 = hash__1 Xor hash__1 << 25
            hash__1 += hash__1 >> 6

            bytes = BitConverter.GetBytes(CUInt(hash__1))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

    Public Class XOR_8

        Public Function Compute(ByVal Bytes As Byte()) As Byte
            Dim result As Byte
            For Each b As Byte In Bytes
                result = result Xor CByte(b)
            Next
            Return CByte(result)
        End Function

    End Class

    Public Class XUM_32

        Private CRC_Table(255) As UInteger
        Private CRC_Polynomial As UInteger = &HEDB88320UI
        Private CRC_InitialValue As UInteger = &HFFFFFFFFUI

        Sub New()
            Dim i As Integer
            Dim j As Integer
            Dim crc As UInteger
            For i = 0 To 255
                crc = CUInt(i)
                For j = 0 To 7
                    If (crc And &H1L) Then
                        crc = (crc >> 1) Xor Me.CRC_Polynomial
                    Else
                        crc = (crc >> 1)
                    End If
                Next j
                Me.CRC_Table(i) = crc
            Next i
        End Sub

        Public Function Compute(ByVal bytes As Byte()) As Byte()
            Dim length As Long = 0
            Dim crc32Value As UInteger = &HFFFFFFFFUI
            Dim elf32Value As UInteger = 0
            Dim crc32Tmp As UInteger = crc32Value
            Dim elf32Tmp As UInteger = elf32Value
            For Each b As Byte In bytes
                crc32Tmp = (crc32Tmp >> 8) Xor Me.CRC_Table((crc32Tmp Xor b) And &HFF)
                elf32Tmp = (elf32Tmp << 4) + b
                Dim ghash As UInteger = elf32Tmp And &HF0000000UI
                elf32Tmp = elf32Tmp Xor (ghash >> 24)
                elf32Tmp = elf32Tmp And Not ghash
                elf32Value = elf32Tmp
                crc32Value = crc32Tmp
                length += 1
            Next
            Dim xum32 As UInteger = CUInt((length << 16) + (length >> 16))
            xum32 = xum32 Xor ((Not crc32Value))
            xum32 = xum32 Xor (elf32Value Mod 997)
            bytes = BitConverter.GetBytes(CUInt(xum32))
            Array.Reverse(bytes)
            Return bytes
        End Function

    End Class

End Namespace

